@@ -32,6 +32,9 @@
 #include "gui.h"
 #include "midi.h"
 #include "player.h"
+#ifdef HAVE_JACK
+#include "jack.h"
+#endif
 
 #define DEFAULT_TEMPO 130
 
@@ -40,6 +43,12 @@ enum {
   MIDIINTERFACES_UNAVAILABLE
 };
 
+enum {
+  EXTERNAL_SYNC_OFF,
+  EXTERNAL_SYNC_MIDI,
+  EXTERNAL_SYNC_JACK
+};
+
 struct editor {
   /* The song currently being edited */
   struct song *song;
@@ -63,6 +72,10 @@ struct editor {
   struct midi *midi;
   /* Player for this editor */
   struct player *player;
+#ifdef HAVE_JACK
+  /* Jack subsystem */
+  struct jack *jack;
+#endif
   /* GConf client for preferences */
   GConfClient *gconfclient;
   guint gconfnotifyid;
@@ -71,6 +84,7 @@ struct editor {
   unsigned int receivemessage;
   unsigned int receivedbytes;
   unsigned char receiveautostop;
+  unsigned int externalsync;
 };
 
 /* Initializes the editor */
@@ -95,6 +109,7 @@ void editor_set_playseq(struct editor *, int);
 void editor_set_position(struct editor *, int);
 void editor_set_block(struct editor *, int);
 void editor_set_line(struct editor *, int);
+void editor_seek(struct editor *, int);
 void editor_set_commandpage(struct editor *, int);
 void editor_set_time(struct editor *, int);
 void editor_set_tempo(struct editor *);
@@ -117,6 +132,7 @@ unsigned int editor_song_block_current_length_get(struct editor *);
 unsigned int editor_song_block_current_tracks_get(struct editor *);
 unsigned int editor_song_block_current_commandpages_get(struct editor *);
 void editor_song_block_current_note_set(struct editor *, unsigned int, unsigned int, unsigned char, unsigned char, unsigned char);
+unsigned char editor_song_block_current_instrument_get(struct editor *, unsigned int, unsigned int);
 void editor_song_block_current_instrument_set(struct editor *, unsigned int, unsigned int, unsigned char);
 void editor_song_block_current_command_set(struct editor *, unsigned int, unsigned int, unsigned int, unsigned char, unsigned char);
 void editor_song_block_current_command_full_set(struct editor *, unsigned int, unsigned int, unsigned int, unsigned char, unsigned char);
@@ -212,12 +228,14 @@ void editor_song_ticksperline_set(struct editor *, unsigned int);
 unsigned int editor_song_ticksperline_get(struct editor *);
 char *editor_song_name_get(struct editor *);
 unsigned int editor_song_sendsync_get(struct editor *);
-  
+
 /* Preferences functions */
 /* Sets the scheduling mode */
 void editor_scheduler_set(struct editor *, unsigned int);
 /* Returns the scheduling mode */
 unsigned int editor_scheduler_get(struct editor *);
+/* Returns the tracker font name */
+char *editor_tracker_font_get(struct editor *);
 /* Enables or disables a MIDI interface by name */
 unsigned int editor_midi_interface_status_set(struct editor *, unsigned int, unsigned int, unsigned int, unsigned int);
 /* Returns the name of the MIDI interface to use */
@@ -237,5 +255,10 @@ void editor_player_pitch_reset(struct editor *);
 void editor_player_trackstatus_create(struct editor *, unsigned int);
 unsigned int editor_player_mode_get(struct editor *);
 void editor_player_play_note(struct editor *, unsigned char, unsigned char);
+void editor_player_set_scheduler(struct editor *);
+unsigned int editor_player_get_external_sync(struct editor *);
+void editor_player_set_external_sync(struct editor *, unsigned int);
+void editor_player_external_sync(struct editor *, unsigned int);
+unsigned int editor_player_available(struct editor *);
 
 #endif
