@@ -120,10 +120,15 @@ enum {
 };
 
 enum {
-  GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_DEVICENAME,
   GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_NAME,
-  GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_INPUT_ENABLED,
-  GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_OUTPUT_ENABLED
+  GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_ENABLED,
+  GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_CLIENT,
+  GUI_PREFERENCES_MIDI_INTERFACES_COLUMN_PORT
+};
+
+enum {
+  GUI_MIDI_INTERFACE_COLUMN_NAME,
+  GUI_MIDI_INTERFACE_COLUMN_NUMBER
 };
 
 /* Opening and closing the GUI */
@@ -154,7 +159,7 @@ void gui_trackvolumes_refresh(struct gui *);
 void gui_preferences_refresh(struct gui *);
 void gui_tracker_refresh(struct gui *);
 void gui_song_refresh(struct gui *);
-void gui_receive_refresh(struct gui *, unsigned int, unsigned int, gboolean);
+void gui_receive_refresh(struct gui *, int, int);
   
 Tracker *gui_get_tracker(struct gui *);
 int gui_get_tracker_position(struct gui *);
