@@ -41,6 +41,8 @@ enum {
 
 /* MIDI data */
 struct midi {
+  /* The editor this stuff is connected to (or NULL) */
+  struct editor *editor;
   /* ALSA MIDI sequencer interface */
   snd_seq_t *seq;
   /* Sequencer client ID */
@@ -121,7 +123,7 @@ struct message {
 };
 
 /* Open the MIDI subsystem */
-struct midi *midi_open(char *, char *, char *, char *);
+struct midi *midi_open(struct editor *, char *, char *, char *, char *);
 /* Close the MIDI subsystem */
 void midi_close(struct midi *);
 /* Subscribe to a client/port */
