@@ -60,7 +60,8 @@ enum {
 enum {
   SCHED_NONE,
   SCHED_RTC,
-  SCHED_NANOSLEEP
+  SCHED_NANOSLEEP,
+  SCHED_EXTERNAL_SYNC
 };
 
 struct player {
@@ -86,12 +87,20 @@ struct player {
   GThread *thread;
   /* Mutex for the player thread */
   GMutex *mutex;
+  /* Cond for external sync */
+  GCond *external_sync;
+  /* External sync tick count */
+  int external_sync_ticks;
   /* Kill player flag (the mutex must be used when accessing) */
   unsigned int killthread;
   /* MIDI subsystem */
   struct midi *midi;
   /* RTC device file descriptor */
   int rtc;
+  /* Obtained RTC frequency */
+  int rtc_frequency;
+  /* RTC periodic interrupts enabled */
+  int rtc_pie;
   /* Editor for this player */
   struct editor *editor;
   /* Indicates whether some tracks are soloed or not */
@@ -101,7 +110,7 @@ struct player {
 /* Plays a song to a MIDI interface witout any scheduling (for export) */
 void player_midi_export(struct song *, struct midi_interface *);
 /* Creates a player for a song */
-struct player *player_open(struct song *, struct editor *, struct midi *, unsigned int);
+struct player *player_open(struct song *, struct editor *, struct midi *);
 /* Closes a player for a song */
 void player_close(struct player *);
 
@@ -112,6 +121,8 @@ void player_stop(struct player *);
 
 /* Plays a note using given instrument on a given channel */
 void player_play_note(struct player *, unsigned int, unsigned char, unsigned char, unsigned char);
+/* Stops notes playing on muted tracks */
+void player_stop_muted(struct player *);
 /* Stops notes playing at the moment */
 void player_stop_notes(struct player *);
 /* Stops all notes */
@@ -129,7 +140,8 @@ void player_set_playseq(struct player *, int);
 void player_set_position(struct player *, int);
 void player_set_block(struct player *, int);
 void player_set_line(struct player *, int);
-  
+void player_set_tick(struct player *, int);
+
 /* Sets whether some tracks are considered soloed or not */
 void player_set_solo(struct player *, unsigned int);
 /* Checks whether some tracks are soloed or not */
@@ -142,4 +154,10 @@ void player_lock(struct player *);
 /* Unlock the player */
 void player_unlock(struct player *);
 
+/* A method to notify the player about an incoming sync signal */
+void player_external_sync(struct player *, unsigned int);
+
+/* Set the scheduler of a player */
+void player_set_scheduler(struct player *, unsigned int);
+
 #endif
