@@ -29,7 +29,7 @@
 #define TRACKER(obj)          GTK_CHECK_CAST (obj, tracker_get_type (), Tracker)
 #define TRACKER_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, tracker_get_type (), TrackerClass)
 #define IS_TRACKER(obj)       GTK_CHECK_TYPE (obj, tracker_get_type ())
-#define TRACKER_CHANNEL_WIDTH 10
+#define TRACKER_CHANNEL_WIDTH 13
 
 typedef struct _Tracker       Tracker;
 typedef struct _TrackerClass  TrackerClass;
@@ -37,12 +37,13 @@ typedef struct _TrackerClass  TrackerClass;
 enum {
   TRACKERCOL_BG,
   TRACKERCOL_BG_CURSOR,
-  TRACKERCOL_BG_MAJHIGH,
-  TRACKERCOL_BG_MINHIGH,
   TRACKERCOL_BG_SELECTION,
   TRACKERCOL_NOTES,	
   TRACKERCOL_BARS,
-  TRACKERCOL_CHANNUMS,
+  TRACKERCOL_CHANNEL_HEADER,
+  TRACKERCOL_CHANNEL_HEADER_MUTE,
+  TRACKERCOL_CHANNEL_HEADER_SOLO,
+  TRACKERCOL_CHANNEL_HEADER_MUTE_SOLO,
   TRACKERCOL_CURSOR,
   TRACKERCOL_LAST
 };
@@ -56,17 +57,17 @@ struct _Tracker {
   int disp_startx;
   int disp_chanwidth;
   int disp_cursor;
+
+  PangoFontDescription *fontdesc;
+  int fontw, fonth, fontc;
   
-  GdkFont *font;
-  int fonth, fontw;
-  int baselineskip;
-  
-  GdkGC *bg_gc, *bg_cursor_gc, *bg_majhigh_gc, *bg_minhigh_gc, *notes_gc, *misc_gc;
+  GdkGC *bg_gc, *bg_cursor_gc, *notes_gc, *misc_gc;
   GdkColor colors[TRACKERCOL_LAST];
   int enable_backing_store;
   GdkPixmap *pixmap;
   guint idle_handler;
   
+  struct song *song;
   struct block *curpattern;
   int cmdpage;
   int patpos, oldpos;
@@ -82,14 +83,18 @@ struct _Tracker {
   
   int mouse_selecting;
   int button;
+  
+  GdkPixmap **fonts_normal;
+  GdkPixmap **fonts_cursor;
+  GdkPixmap **fonts_select;
 };
 
 struct _TrackerClass {
   GtkWidgetClass parent_class;
   
-  void (*patpos)(Tracker *t, int patpos, int patlen, int disprows);
-  void (*xpanning)(Tracker *t, int leftchan, int numchans, int dispchans);
-  void (*blockmark_set)(Tracker *t, int state);
+  void (*patpos)(gpointer, int, int, int, gpointer);
+  void (*xpanning)(gpointer, int, int, int, gpointer);
+  void (*blockmark_set)(gpointer, int, gpointer);
 };
 
 extern const char * const notenames[128];
@@ -115,6 +120,7 @@ void           	tracker_step_cursor_channel (Tracker *t, int direction);
 void           	tracker_set_patpos          (Tracker *t, int row);
 void           	tracker_step_cursor_row     (Tracker *t, int direction);
 
+void           	tracker_set_song            (Tracker *t, struct song *);
 void           	tracker_set_pattern         (Tracker *t, struct block *pattern);
 void           	tracker_set_xpanning        (Tracker *t, int left_channel);
 
