@@ -70,6 +70,14 @@ void block_set_note(struct block *block, unsigned int line, unsigned int track,
   }
 }
 
+/* Gets an instrument from a block */
+unsigned char block_get_instrument(struct block *block, unsigned int line, unsigned int track)
+{
+  nullcheck_int(block, block_set_instrument);
+
+  return block->notes[2 * (block->tracks * line + track) + 1];
+}
+
 /* Sets an instrument in a block */
 void block_set_instrument(struct block *block, unsigned int line, unsigned int track, unsigned char instrument)
 {
