@@ -503,14 +503,39 @@ void on_menuitem_midi_resetpitch_activate(GtkMenuItem * menuitem, gpointer user_
   editor_player_pitch_reset(gui->editor);
 }
 
-void on_menuitem_midi_recordcontrollers_activate(GtkMenuItem * menuitem, gpointer user_data)
+void on_menuitem_settings_sendmidisync_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
 
-  if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget(gui->xml, "menuitem_midi_recordcontrollers"))))
-    gui->editor->recordcontrollers = 1;
-  else
-    gui->editor->recordcontrollers = 0;
+  editor_song_sendsync_set(gui->editor, gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) ? 1 : 0);
+}
+
+void on_menuitem_settings_recordcontrollers_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+
+  gui->editor->recordcontrollers = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) ? 1 : 0;
+}
+
+void on_menuitem_settings_externalsync_off_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+
+  editor_player_set_external_sync(gui->editor, EXTERNAL_SYNC_OFF);
+}
+
+void on_menuitem_settings_externalsync_midi_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+
+  editor_player_set_external_sync(gui->editor, EXTERNAL_SYNC_MIDI);
+}
+
+void on_menuitem_settings_externalsync_jack_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+
+  editor_player_set_external_sync(gui->editor, EXTERNAL_SYNC_JACK);
 }
 
 void on_menuitem_settings_preferences_activate(GtkMenuItem * menuitem, gpointer user_data)
@@ -553,7 +578,6 @@ void on_button_playsong_clicked(GtkButton * button, gpointer user_data)
   struct gui *gui = (struct gui *)user_data;
 
   editor_player_start(gui->editor, MODE_PLAY_SONG, 0);
-  gui_statusbar_refresh(gui);
 }
 
 void on_button_playblock_clicked(GtkButton * button, gpointer user_data)
@@ -561,25 +585,20 @@ void on_button_playblock_clicked(GtkButton * button, gpointer user_data)
   struct gui *gui = (struct gui *)user_data;
 
   editor_player_start(gui->editor, MODE_PLAY_BLOCK, 0);
-  gui_statusbar_refresh(gui);
 }
 
 void on_button_contsong_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  if (editor_player_mode_get(gui->editor) != MODE_PLAY_SONG) {
+  if (editor_player_mode_get(gui->editor) != MODE_PLAY_SONG)
     editor_player_start(gui->editor, MODE_PLAY_SONG, 1);
-    gui_statusbar_refresh(gui);
-  }
 }
 
 void on_button_contblock_clicked(GtkButton * button, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
-  if (editor_player_mode_get(gui->editor) != MODE_PLAY_BLOCK) {
+  if (editor_player_mode_get(gui->editor) != MODE_PLAY_BLOCK)
     editor_player_start(gui->editor, MODE_PLAY_BLOCK, 1);
-    gui_statusbar_refresh(gui);
-  }
 }
 
 void on_button_stop_clicked(GtkButton * button, gpointer user_data)
@@ -1128,15 +1147,11 @@ void on_spinbutton_instrument_changed(GtkEditable * editable, gpointer user_data
 
 gint on_spinbutton_instrument_output(GtkSpinButton * spin_button, gpointer user_data)
 {
-  gchar v[2];
+  gchar v[3];
   int i = (int)spin_button->adjustment->value;
 
-  /* Convert the instrument number to 0-9 / A-Z */
-  if (i < 10)
-    v[0] = '0' + i;
-  else
-    v[0] = 'A' + (i - 10);
-  v[1] = 0;
+  /* Convert the instrument number to hex */
+  snprintf(v, 3, "%.2X", i);
 
   /* Only update the GtkEntry if the value was changed */
   if (strcmp(v, gtk_entry_get_text(GTK_ENTRY(spin_button))))
@@ -1149,20 +1164,16 @@ gint on_spinbutton_instrument_input(GtkSpinButton * spin_button, gpointer arg1,
 {
   const gchar *c = gtk_entry_get_text(GTK_ENTRY(spin_button));
   gdouble *new_val = arg1;
-
-  if (c == NULL || strlen(c) > 1)
+  char *err;
+  
+  if (c == NULL)
     return GTK_INPUT_ERROR;
 
-  /* Convert the 0-9 / A-Z string to an integer */
-  if (c[0] >= '0' && c[0] <= '9')
-    *new_val = (gdouble) (c[0] - '0');
-  else if (c[0] >= 'A' && c[0] <= 'Z')
-    *new_val = (gdouble) ((c[0] - 'A') + 10);
-  else if (c[0] >= 'a' && c[0] <= 'z')
-    *new_val = (gdouble) ((c[0] - 'a') + 10);
-  else
+  /* Convert the hex string to an integer */
+  *new_val = strtol(c, &err, 16);
+  if (*err != 0)
     return GTK_INPUT_ERROR;
-
+  
   return TRUE;
 }
 
@@ -1179,10 +1190,7 @@ void on_togglebutton_trackvolumes_mute_toggled(GtkToggleButton * togglebutton, g
   struct gui *gui = (struct gui *)user_data;
   int track = (int)g_object_get_data(G_OBJECT(togglebutton), "track");
 
-  if (gtk_toggle_button_get_active(togglebutton))
-    editor_song_track_mute_set(gui->editor, track, 1);
-  else
-    editor_song_track_mute_set(gui->editor, track, 0);
+  editor_song_track_mute_set(gui->editor, track, gtk_toggle_button_get_active(togglebutton) ? 1 : 0);
 }
 
 void on_togglebutton_trackvolumes_solo_toggled(GtkToggleButton * togglebutton, gpointer user_data)
@@ -1190,10 +1198,7 @@ void on_togglebutton_trackvolumes_solo_toggled(GtkToggleButton * togglebutton, g
   struct gui *gui = (struct gui *)user_data;
   int track = (int)g_object_get_data(G_OBJECT(togglebutton), "track");
 
-  if (gtk_toggle_button_get_active(togglebutton))
-    editor_song_track_solo_set(gui->editor, track, 1);
-  else
-    editor_song_track_solo_set(gui->editor, track, 0);
+  editor_song_track_solo_set(gui->editor, track, gtk_toggle_button_get_active(togglebutton) ? 1 : 0);
 }
 
 void on_entry_trackvolumes_name_changed(GtkEditable * widget, gpointer user_data)
@@ -1419,15 +1424,6 @@ void on_button_changeinstrument_change_clicked(GtkButton * button, gpointer user
   tracker_redraw(tracker);
 }
 
-void on_checkbutton_sendsync_toggled(GtkToggleButton * togglebutton, gpointer user_data)
-{
-  struct gui *gui = (struct gui *)user_data;
-  if (gtk_toggle_button_get_active(togglebutton))
-    editor_song_sendsync_set(gui->editor, 1);
-  else
-    editor_song_sendsync_set(gui->editor, 0);
-}
-
 void on_checkbutton_message_autosend_toggled(GtkToggleButton * togglebutton, gpointer user_data)
 {
   struct gui *gui = (struct gui *)user_data;
@@ -1884,12 +1880,13 @@ void on_cell_blocklist_edited(GtkCellRendererText * cell, gchar * path_string, g
   case GUI_BLOCKLIST_COLUMN_TRACKS:
     {
       editor_song_block_current_tracks_set(gui->editor, value);
-      gui_tracker_refresh(gui);
 
       /* No need to call editor_player_trackstatus_create since
 	 editor_song_maxtracks_check does the necessary thing automatically */
       if (editor_song_maxtracks_check(gui->editor))
 	gui_trackvolumes_refresh(gui);
+
+      gui_tracker_refresh(gui);
       break;
     }
   case GUI_BLOCKLIST_COLUMN_LENGTH:
@@ -2050,3 +2047,72 @@ void on_dialog_receive_delete_event(GtkWidget * widget, GdkEvent * event, gpoint
 {
   on_button_receive_close_clicked(NULL, user_data);
 }
+
+void on_vscrollbar_tracker_value_changed(GtkRange * range, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  int line = (int)(gtk_range_get_value(range) * (editor_song_block_current_length_get(gui->editor) + tracker->disp_rows));
+  editor_set_line(gui->editor, line);
+}
+
+void on_hscrollbar_tracker_value_changed(GtkRange * range, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  int left = (int)(gtk_range_get_value(range) * editor_song_block_current_tracks_get(gui->editor));
+  tracker_set_xpanning(tracker, left);
+}
+
+void on_tracker_patpos(GtkWidget * widget, int current, int length, int visible, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+    
+  GtkRange *r = GTK_RANGE(glade_xml_get_widget(gui->xml, "vscrollbar_tracker"));
+  GtkAdjustment *a = gtk_range_get_adjustment(r);
+  a->value = (gdouble)current / (length + visible - 1);
+  a->lower = (gdouble)0;
+  a->upper = (gdouble)1;
+  a->step_increment = (gdouble)1 / (length + visible - 1);
+  a->page_increment = (gdouble)8 / (length + visible - 1);
+  a->page_size = (gdouble)visible / (length + visible - 1);
+  gtk_adjustment_changed(a);
+}
+
+void on_tracker_xpanning(GtkWidget * widget, int current, int tracks, int visible, gpointer user_data)
+{
+  struct gui *gui = (struct gui *)user_data;
+    
+  GtkRange *r = GTK_RANGE(glade_xml_get_widget(gui->xml, "hscrollbar_tracker"));
+  GtkAdjustment *a = gtk_range_get_adjustment(r);
+  a->value = (gdouble)current / tracks;
+  a->lower = (gdouble)0;
+  a->upper = (gdouble)1;
+  a->step_increment = (gdouble)1 / tracks;
+  a->page_increment = (gdouble)4 / tracks;
+  a->page_size = (gdouble)visible / tracks;
+  gtk_adjustment_changed(a);
+}
+
+gboolean on_tracker_scroll_event (GtkWidget *widget, GdkEventScroll *event, gpointer user_data) {
+  struct gui *gui = (struct gui *)user_data;
+  Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+
+  switch (event->direction) {
+  case GDK_SCROLL_UP:
+    editor_set_line(gui->editor, gui->editor->line - 4);
+    return TRUE;
+  case GDK_SCROLL_DOWN:
+    editor_set_line(gui->editor, gui->editor->line + 4);
+    return TRUE;
+  case GDK_SCROLL_LEFT:
+    tracker_step_cursor_item(tracker, -2);
+    return TRUE;
+  case GDK_SCROLL_RIGHT:
+    tracker_step_cursor_item(tracker, 2);
+    return TRUE;
+  default:
+    break;
+  }
+  return FALSE;
+}
