@@ -27,6 +27,9 @@
 #include "midi.h"
 #include "player.h"
 #include "callbacks.h"
+#ifdef HAVE_JACK
+#include "jack.h"
+#endif
 
 /* Initializes the editor */
 struct editor *editor_open()
@@ -47,7 +50,12 @@ struct editor *editor_open()
   /* Create a mutex for locking the song */
   editor->songmutex = g_mutex_new();
 
-  /* Open a GUI file */
+#ifdef HAVE_JACK
+  /* Open the Jack subsystem */
+  editor->jack = jack_open(editor);
+#endif
+
+  /* Open the GUI subsystem */
   editor->gui = gui_open(editor);
   if (editor->gui == NULL) {
     editor_close(editor);
@@ -55,7 +63,7 @@ struct editor *editor_open()
   }
 
   /* Open the MIDI subsystem */
-  editor->midi = midi_open(editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_AVAILABLE), editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_AVAILABLE), editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_UNAVAILABLE), editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_UNAVAILABLE));
+  editor->midi = midi_open(editor, editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_AVAILABLE), editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_AVAILABLE), editor_midi_interface_get(editor, DIRECTION_IN, MIDIINTERFACES_UNAVAILABLE), editor_midi_interface_get(editor, DIRECTION_OUT, MIDIINTERFACES_UNAVAILABLE));
 
   /* Some interfaces may have been removed so refresh preferences */
   editor_midi_interface_set_from_current(editor);
@@ -123,7 +131,7 @@ void editor_song_open(struct editor *editor, char *filename)
     path = g_realloc(path, strlen(path) + 2);
     path[strlen(path) + 1] = 0;
     path[strlen(path)] = '/';
-    gconf_client_set_string(editor->gconfclient, "/apps/tutka/songpath", path, NULL);
+    gconf_client_set_string(editor->gconfclient, "/apps/tutka/song_path", path, NULL);
     g_free(path);
   } else {
     /* Figure out default MIDI interface number for new instruments */
@@ -146,7 +154,8 @@ void editor_song_open(struct editor *editor, char *filename)
   editor->cmdpage = 0;
 
   /* Open the player */
-  editor->player = player_open(editor->song, editor, editor->midi, editor_scheduler_get(editor));
+  editor->player = player_open(editor->song, editor, editor->midi);
+  player_set_scheduler(editor->player, editor_scheduler_get(editor));
 
   /* Refresh the GUI */
   gui_refresh_all(editor->gui);
@@ -161,7 +170,13 @@ void editor_close(struct editor *editor)
   /* Close the player */
   if (editor->player != NULL)
     player_close(editor->player);
-  
+
+#ifdef HAVE_JACK
+  /* Close Jack */
+  if (editor->jack != NULL)
+    jack_close(editor->jack);
+#endif
+
   /* Close MIDI */
   if (editor->midi != NULL)
     midi_close(editor->midi);
@@ -242,10 +257,11 @@ void editor_song_midi_interface_names_set(struct editor *editor)
 }
 
 /* Checks whether there have been changes in MIDI interfaces */
-void editor_midi_status_check(struct editor *editor) {
+void editor_midi_status_check(struct editor *editor)
+{
   nullcheck_void(editor, editor_midi_status_check);
-  
-  if(!editor->midi->changed)
+
+  if (!editor->midi->changed)
     return;
 
   /* Store the old MIDI interface names */
@@ -265,14 +281,14 @@ void editor_midi_status_check(struct editor *editor) {
 
   /* The player is free to go */
   player_unlock(editor->player);
-  
+
   /* Refresh preferences and instrument windows */
   gui_instrument_refresh(editor->gui);
   gui_preferences_refresh(editor->gui);
-  
+
   /* Update the preferences */
   editor_midi_interface_set_from_current(editor);
-  
+
   editor->midi->changed = 0;
 }
 
@@ -387,6 +403,118 @@ void editor_set_line(struct editor *editor, int line)
   }
 }
 
+/* Seeks to a certain number of ticks in a song and refreshes the GUI accordingly */
+void editor_seek(struct editor *editor, int ticks)
+{
+  nullcheck_void(editor, editor_seek);
+
+  unsigned int section, playseq, position, block, line, tick, tpl, i, j;
+
+  tpl = editor->song->ticksperline;
+  section = 0;
+  playseq = editor->song->sections[section];
+  position = 0;
+  block = editor->song->playseqs[playseq]->blocknumbers[position];
+  line = 0;
+  tick = 0;
+
+  while (ticks > 0) {
+    struct block *b = editor->song->blocks[block];
+    unsigned char postcommand = 0, postvalue = 0, changeblock = 0;
+
+    /* Look for commands that alter the playing position */
+    for (i = 0; i < b->tracks; i++) {
+      for (j = 0; j < b->commandpages; j++) {
+	unsigned char command = b->commands[j * b->tracks * b->length * 2 + (b->tracks * line + i) * 2];
+	unsigned char value = b->commands[j * b->tracks * b->length * 2 + (b->tracks * line + i) * 2 + 1];
+
+	if (tick == tpl - 1 && (command == COMMAND_END_BLOCK || command == COMMAND_PLAYSEQ_POSITION || command == COMMAND_TPL)) {
+	  postcommand = command;
+	  postvalue = value;
+	}
+      }
+    }
+
+    tick++;
+    tick %= tpl;
+
+    /* Ticks per line ticks have passed, move on */
+    if (tick == 0) {
+      line++;
+
+      /* Check if a playing position altering command was used */
+      switch (postcommand) {
+      case COMMAND_END_BLOCK:
+	line = postvalue;
+	position++;
+	if (position >= editor->song->playseqs[playseq]->length) {
+	  position = 0;
+	  section++;
+	  if (section >= editor->song->numsections)
+	    section = 0;
+	}
+	changeblock = 1;
+	break;
+      case COMMAND_PLAYSEQ_POSITION:
+	line = 0;
+	position = postvalue;
+	if (position >= editor->song->playseqs[playseq]->length) {
+	  position = 0;
+	  section++;
+	  if (section >= editor->song->numsections)
+	    section = 0;
+	}
+	changeblock = 1;
+	break;
+      case COMMAND_TPL:
+	tpl = postvalue;
+	/* Fall through to default: */
+      default:
+	if (line >= b->length) {
+	  line = 0;
+	  position++;
+	  if (position >= editor->song->playseqs[playseq]->length) {
+	    position = 0;
+	    section++;
+	    if (section >= editor->song->numsections)
+	      section = 0;
+	  }
+	  changeblock = 1;
+	}
+	break;
+      }
+
+      if (changeblock == 1) {
+	if (section >= editor->song->numsections)
+	  section = 0;
+	playseq = editor->song->sections[section];
+
+	if (position >= editor->song->playseqs[playseq]->length)
+	  position = 0;
+          
+        block = editor->song->playseqs[playseq]->blocknumbers[position];
+      }
+    }
+
+    ticks--;
+  }
+  
+  /* Set the position in editor, player and gui */
+  editor->section = section;
+  editor->playseq = playseq;
+  editor->position = position;
+  editor->block = block;
+  editor->line = line;
+  player_set_section(editor->player, section);
+  player_set_playseq(editor->player, playseq);
+  player_set_position(editor->player, position);
+  player_set_block(editor->player, block);
+  player_set_line(editor->player, line);
+  player_set_tick(editor->player, tick);
+  editor_refresh_playseq_and_block(editor);
+  gui_set_refresh(editor->gui, GUI_REFRESH_SECTIONLIST | GUI_REFRESH_PLAYSEQLIST | GUI_REFRESH_PLAYSEQ_RECREATE | GUI_REFRESH_PLAYSEQ | GUI_REFRESH_BLOCKLIST | GUI_REFRESH_TRACKER | GUI_REFRESH_INFO);
+}
+
 /* Changes the position of the editor and refreshes the GUI accordingly */
 void editor_set_commandpage(struct editor *editor, int cmdpage)
 {
@@ -438,11 +566,10 @@ void editor_refresh_playseq_and_block(struct editor *editor)
   editor->playseq = song->sections[editor->section];
 
   /* Bounds checking */
-  if (editor->position >= song->playseqs[editor->playseq]->length) {
+  if (editor->position >= song->playseqs[editor->playseq]->length)
     editor->position = 0;
-    editor_refresh_playseq_and_block(editor);
-  } else
-    editor->block = song->playseqs[editor->playseq]->blocknumbers[editor->position];
+
+  editor->block = song->playseqs[editor->playseq]->blocknumbers[editor->position];
 
   /* If called from GUI set player position, always refresh GUI */
   if (g_thread_self() == editor->thread)
@@ -712,6 +839,14 @@ void editor_song_block_current_note_set(struct editor *editor, unsigned int line
   g_mutex_unlock(editor->songmutex);
 }
 
+/* Gets an instrument from the current block */
+unsigned char editor_song_block_current_instrument_get(struct editor *editor, unsigned int line, unsigned int track)
+{
+  nullcheck_int(editor, editor_song_block_current_instrument_get);
+
+  return block_get_instrument(editor->song->blocks[editor->block], line, track);
+}
+
 /* Sets an instrument in the current block */
 void editor_song_block_current_instrument_set(struct editor *editor, unsigned int line, unsigned int track, unsigned char instrument)
 {
@@ -884,6 +1019,10 @@ void editor_song_track_mute_set(struct editor *editor, unsigned int track, unsig
   g_mutex_lock(editor->songmutex);
   editor->song->tracks[track]->mute = mute;
   g_mutex_unlock(editor->songmutex);
+
+  player_lock(editor->player);
+  player_stop_muted(editor->player);
+  player_unlock(editor->player);
 }
 
 /* Returns the muting of a track */
@@ -907,6 +1046,10 @@ void editor_song_track_solo_set(struct editor *editor, unsigned int track, unsig
   else
     player_check_solo(editor->player);
   g_mutex_unlock(editor->songmutex);
+
+  player_lock(editor->player);
+  player_stop_muted(editor->player);
+  player_unlock(editor->player);
 }
 
 /* Returns the soloing of a track */
@@ -928,6 +1071,9 @@ void editor_song_track_name_set(struct editor *editor, unsigned int track, char
   else
     editor->song->tracks[track]->name = NULL;
   g_mutex_unlock(editor->songmutex);
+
+  /* Refresh the tracker */
+  gui_tracker_refresh(editor->gui);
 }
 
 /* Returns the name of a track */
@@ -1215,7 +1361,7 @@ void editor_song_block_current_delete(struct editor *editor)
 
   /* Make sure the current block is not beyond limits */
   if (editor->block >= editor->song->numblocks)
-    editor->block = editor->song->numblocks - 1;
+    editor_set_block(editor, editor->song->numblocks - 1);
   g_mutex_unlock(editor->songmutex);
 }
 
@@ -1281,7 +1427,7 @@ void editor_song_save(struct editor *editor, char *filename)
   path = g_realloc(path, strlen(path) + 2);
   path[strlen(path) + 1] = 0;
   path[strlen(path)] = '/';
-  gconf_client_set_string(editor->gconfclient, "/apps/tutka/songpath", path, NULL);
+  gconf_client_set_string(editor->gconfclient, "/apps/tutka/song_path", path, NULL);
   g_free(path);
 }
 
@@ -1406,7 +1552,7 @@ void editor_song_message_save(struct editor *editor, unsigned int message, char
   path = g_realloc(path, strlen(path) + 2);
   path[strlen(path) + 1] = 0;
   path[strlen(path)] = '/';
-  gconf_client_set_string(editor->gconfclient, "/apps/tutka/messagepath", path, NULL);
+  gconf_client_set_string(editor->gconfclient, "/apps/tutka/message_path", path, NULL);
   g_free(path);
 }
 
@@ -1426,7 +1572,7 @@ void editor_song_message_load(struct editor *editor, unsigned int message, char
   path = g_realloc(path, strlen(path) + 2);
   path[strlen(path) + 1] = 0;
   path[strlen(path)] = '/';
-  gconf_client_set_string(editor->gconfclient, "/apps/tutka/messagepath", path, NULL);
+  gconf_client_set_string(editor->gconfclient, "/apps/tutka/message_path", path, NULL);
   g_free(path);
 }
 
@@ -1451,6 +1597,10 @@ void editor_song_mute(struct editor *editor)
     for (i = 0; i < editor->song->maxtracks; i++)
       editor->song->tracks[i]->mute = 1;
   g_mutex_unlock(editor->songmutex);
+
+  player_lock(editor->player);
+  player_stop_muted(editor->player);
+  player_unlock(editor->player);
 }
 
 /* Sets the playing sequence of the current section */
@@ -1713,7 +1863,8 @@ void editor_scheduler_set(struct editor *editor, unsigned int mode)
 {
   nullcheck_void(editor, editor_scheduler_set);
 
-  gconf_client_set_int(editor->gconfclient, "/apps/tutka/scheduler", mode, NULL);
+  gconf_client_set_int(editor->gconfclient, "/apps/tutka/scheduling_mode", mode, NULL);
+  editor_player_set_scheduler(editor);
 }
 
 /* Returns the scheduling mode */
@@ -1721,7 +1872,7 @@ unsigned int editor_scheduler_get(struct editor *editor)
 {
   nullcheck_int(editor, editor_scheduler_get);
 
-  int scheduler = gconf_client_get_int(editor->gconfclient, "/apps/tutka/scheduler", NULL);
+  int scheduler = gconf_client_get_int(editor->gconfclient, "/apps/tutka/scheduling_mode", NULL);
 
   /* For backwards compatibility: don't allow the scheduler to be 0 */
   if (scheduler == SCHED_NONE) {
@@ -1732,6 +1883,14 @@ unsigned int editor_scheduler_get(struct editor *editor)
   return scheduler;
 }
 
+/* Returns the tracker font name */
+char *editor_tracker_font_get(struct editor *editor)
+{
+  nullcheck_pointer(editor, editor_tracker_font_get);
+
+  return gconf_client_get_string(editor->gconfclient, "/apps/tutka/tracker_font", NULL);
+}
+
 /* Sets the names of the MIDI interfaces to enable separated by , */
 void editor_midi_interface_set(struct editor *editor, char *midi_interface, unsigned int direction, unsigned int mode)
 {
@@ -1739,14 +1898,14 @@ void editor_midi_interface_set(struct editor *editor, char *midi_interface, unsi
 
   if (mode == MIDIINTERFACES_AVAILABLE) {
     if (direction == DIRECTION_IN)
-      gconf_client_set_string(editor->gconfclient, "/apps/tutka/midiinterfaces/in", midi_interface, NULL);
+      gconf_client_set_string(editor->gconfclient, "/apps/tutka/midi_interfaces/in", midi_interface, NULL);
     else
-      gconf_client_set_string(editor->gconfclient, "/apps/tutka/midiinterfaces/out", midi_interface, NULL);
+      gconf_client_set_string(editor->gconfclient, "/apps/tutka/midi_interfaces/out", midi_interface, NULL);
   } else {
     if (direction == DIRECTION_IN)
-      gconf_client_set_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailablein", midi_interface, NULL);
+      gconf_client_set_string(editor->gconfclient, "/apps/tutka/midi_interfaces/unavailable_in", midi_interface, NULL);
     else
-      gconf_client_set_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailableout", midi_interface, NULL);
+      gconf_client_set_string(editor->gconfclient, "/apps/tutka/midi_interfaces/unavailable_out", midi_interface, NULL);
   }
 }
 
@@ -1757,26 +1916,27 @@ char *editor_midi_interface_get(struct editor *editor, unsigned int direction, u
 
   if (mode == MIDIINTERFACES_AVAILABLE) {
     if (direction == DIRECTION_IN)
-      return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/in", NULL);
+      return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midi_interfaces/in", NULL);
     else
-      return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/out", NULL);
+      return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midi_interfaces/out", NULL);
   } else {
     if (direction == DIRECTION_IN)
-      return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailablein", NULL);
+      return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midi_interfaces/unavailable_in", NULL);
     else
-      return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailableout", NULL);
+      return gconf_client_get_string(editor->gconfclient, "/apps/tutka/midi_interfaces/unavailable_out", NULL);
   }
 }
 
 /* Removes the last character from a string if the string is not null */
-static char *string_shorten(char *in) {
+static char *string_shorten(char *in)
+{
   if (in == NULL)
     return (char *)calloc(1, sizeof(char));
-  
+
   int l = strlen(in);
   if (l == 0)
     return (char *)calloc(1, sizeof(char));
-    
+
   char *out = (char *)calloc(l, sizeof(char));
   memcpy(out, in, l - 1);
   return out;
@@ -1821,7 +1981,7 @@ unsigned int editor_midi_interface_status_set(struct editor *editor, unsigned in
 
   /* Refresh the instrument window */
   gui_instrument_refresh(editor->gui);
-  
+
   /* Update the preferences */
   editor_midi_interface_set_from_current(editor);
 
@@ -1833,7 +1993,7 @@ char *editor_song_path_get(struct editor *editor)
 {
   nullcheck_pointer(editor, editor_song_path_get);
 
-  return gconf_client_get_string(editor->gconfclient, "/apps/tutka/songpath", NULL);
+  return gconf_client_get_string(editor->gconfclient, "/apps/tutka/song_path", NULL);
 }
 
 /* Returns the message path from the preferences */
@@ -1841,7 +2001,7 @@ char *editor_message_path_get(struct editor *editor)
 {
   nullcheck_pointer(editor, editor_message_path_get);
 
-  return gconf_client_get_string(editor->gconfclient, "/apps/tutka/messagepath", NULL);
+  return gconf_client_get_string(editor->gconfclient, "/apps/tutka/message_path", NULL);
 }
 
 /* Starts the player */
@@ -1850,6 +2010,7 @@ void editor_player_start(struct editor *editor, unsigned int mode, int cont)
   nullcheck_void(editor, editor_player_start);
 
   player_start(editor->player, mode, editor->section, editor->position, editor->block, cont);
+  gui_statusbar_refresh(editor->gui);
 }
 
 /* Stops the player */
@@ -1857,7 +2018,10 @@ void editor_player_stop(struct editor *editor)
 {
   nullcheck_void(editor, editor_player_stop);
 
+  if (editor->externalsync)
+    player_external_sync(editor->player, 0);
   player_stop(editor->player);
+  gui_info_refresh(editor->gui);
 }
 
 /* Stops all notes playing */
@@ -1897,5 +2061,54 @@ void editor_player_play_note(struct editor *editor, unsigned char note, unsigned
 {
   nullcheck_void(editor, editor_player_play_note);
 
+  player_lock(editor->player);
   player_play_note(editor->player, editor->instrument, editor->octave * 12 + note, 127, track);
+  player_unlock(editor->player);
+}
+
+void editor_player_set_scheduler(struct editor *editor)
+{
+  nullcheck_void(editor, editor_player_set_scheduler);
+
+  /* It's possible there is no player yet */
+  if (editor->player == NULL)
+    return;
+
+  if (editor->externalsync != EXTERNAL_SYNC_OFF)
+    player_set_scheduler(editor->player, SCHED_EXTERNAL_SYNC);
+  else {
+    player_set_scheduler(editor->player, editor_scheduler_get(editor));
+    /* Sync once so the sync wait is broken */
+    player_external_sync(editor->player, 0);
+  }
+}
+
+
+unsigned int editor_player_get_external_sync(struct editor *editor)
+{
+  nullcheck_int(editor, editor_player_get_external_sync);
+
+  return editor->externalsync;
+}
+
+void editor_player_set_external_sync(struct editor *editor, unsigned int sync)
+{
+  nullcheck_void(editor, editor_player_set_external_sync);
+
+  editor->externalsync = sync;
+  editor_player_set_scheduler(editor);
+}
+
+void editor_player_external_sync(struct editor *editor, unsigned int ticks)
+{
+  nullcheck_void(editor, editor_player_external_sync);
+
+  player_external_sync(editor->player, ticks);
+}
+
+unsigned int editor_player_available(struct editor *editor)
+{
+  nullcheck_int(editor, editor_player_available);
+
+  return editor->player != NULL ? 1 : 0;
 }
