@@ -34,6 +34,7 @@
 #include "player.h"
 #include "trackerwidget.h"
 #include "cellrendererspin.h"
+#include "prefix.h"
 
 #define new_column gtk_tree_view_column_new_with_attributes
 
@@ -83,7 +84,9 @@ struct gui *gui_open(struct editor *editor)
   GtkListStore *store;
   GdkPixbuf *pixbuf;
   GtkWidget *w;
-  char *songpath, *messagepath;
+  GtkIconTheme *theme;
+  Tracker *tracker;
+  char *songpath, *messagepath, *fontname;
 
   gui->editor = editor;
 
@@ -91,7 +94,7 @@ struct gui *gui_open(struct editor *editor)
   gui->refresh_mutex = g_mutex_new();
 
   /* Load the GUI from a GLADE file and connect signals */
-  gui->xml = glade_xml_new(PACKAGE_DATA_DIR "/glade/tutka.glade", NULL, NULL);
+  gui->xml = glade_xml_new(BR_DATADIR("/tutka/glade/tutka.glade"), NULL, NULL);
   if (gui->xml == NULL) {
     gui_close(gui);
     return NULL;
@@ -274,10 +277,25 @@ struct gui *gui_open(struct editor *editor)
   }
 
   /* Add icons */
-  pixbuf = gdk_pixbuf_new_from_file(PACKAGE_PIXMAP_DIR "/tutka.png", NULL);
+  theme = gtk_icon_theme_get_default();
+  gtk_icon_theme_append_search_path(theme, BR_DATADIR("/pixmaps"));
+  pixbuf = gtk_icon_theme_load_icon(theme, "tutka", 48, 0, NULL);
   if (pixbuf != NULL)
     gtk_window_set_default_icon_list(g_list_append(NULL, pixbuf));
 
+  /* Set tracker properties */
+  tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
+  fontname = editor_tracker_font_get(gui->editor);
+  if (fontname != NULL && strlen(fontname) > 0)
+    tracker_set_font(tracker, fontname);
+  else
+    tracker_set_font(tracker, "Monospace 9");
+
+#ifdef HAVE_JACK
+  if (editor->jack == NULL)
+#endif
+    gtk_widget_hide(glade_xml_get_widget(gui->xml, "menuitem_settings_externalsync_jack"));
+
   return gui;
 }
 
@@ -312,6 +330,7 @@ void gui_refresh_all(struct gui *gui)
   Tracker *tracker = (Tracker *) glade_xml_get_widget(gui->xml, "tracker");
 
   tracker_reset(tracker);
+  tracker_set_song(tracker, gui->editor->song);
   tracker_set_pattern(tracker, editor_song_block_current_get(gui->editor));
   tracker_set_num_channels(tracker, editor_song_block_current_tracks_get(gui->editor));
 
@@ -492,6 +511,7 @@ void gui_tracker_refresh(struct gui *gui)
     gui->editor->cmdpage = cmdpages - 1;
 
   tracker_set_num_channels(tracker, editor_song_block_current_tracks_get(gui->editor));
+  tracker_set_song(tracker, gui->editor->song);
   tracker_set_pattern(tracker, editor_song_block_current_get(gui->editor));
   tracker_set_patpos(tracker, gui->editor->line);
   tracker_set_cmdpage(tracker, gui->editor->cmdpage);
@@ -501,11 +521,11 @@ void gui_tracker_refresh(struct gui *gui)
 void gui_song_refresh(struct gui *gui)
 {
   GtkEntry *entry = GTK_ENTRY(glade_xml_get_widget(gui->xml, "entry_songname"));
-  GtkToggleButton *togglebutton = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui->xml, "checkbutton_sendsync"));
+  GtkCheckMenuItem *menuitem = GTK_CHECK_MENU_ITEM(glade_xml_get_widget(gui->xml, "menuitem_settings_sendmidisync"));
   char *name;
 
   g_signal_handlers_block_by_func(entry, on_entry_songname_changed, gui);
-  g_signal_handlers_block_by_func(togglebutton, on_checkbutton_sendsync_toggled, gui);
+  g_signal_handlers_block_by_func(menuitem, on_menuitem_settings_sendmidisync_activate, gui);
 
   name = editor_song_name_get(gui->editor);
   /* Refresh the song name widget */
@@ -516,12 +536,12 @@ void gui_song_refresh(struct gui *gui)
 
   /* Refresh the send sync widget */
   if (editor_song_sendsync_get(gui->editor))
-    gtk_toggle_button_set_active(togglebutton, TRUE);
+    gtk_check_menu_item_set_active(menuitem, TRUE);
   else
-    gtk_toggle_button_set_active(togglebutton, FALSE);
+    gtk_check_menu_item_set_active(menuitem, FALSE);
 
   g_signal_handlers_unblock_by_func(entry, on_entry_songname_changed, gui);
-  g_signal_handlers_unblock_by_func(togglebutton, on_checkbutton_sendsync_toggled, gui);
+  g_signal_handlers_unblock_by_func(menuitem, on_menuitem_settings_sendmidisync_activate, gui);
 }
 
 /* Refresh widgets related to the properties of the current instrument */
@@ -991,11 +1011,11 @@ void gui_trackvolumes_refresh(struct gui *gui)
     x = gtk_entry_new();
     gtk_entry_set_width_chars(GTK_ENTRY(x), 7);
     g_object_set_data(G_OBJECT(x), "track", (gpointer) i);
-    g_signal_connect(G_OBJECT(x), "changed", G_CALLBACK(on_entry_trackvolumes_name_changed), gui);
     if (name != NULL)
       gtk_entry_set_text(GTK_ENTRY(x), name);
     else
       gtk_entry_set_text(GTK_ENTRY(x), "");
+    g_signal_connect(G_OBJECT(x), "changed", G_CALLBACK(on_entry_trackvolumes_name_changed), gui);
     gtk_table_attach(GTK_TABLE(table), x, i, i + 1, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 2, 4);
   }
 
