@@ -32,7 +32,7 @@ struct instrument *instrument_alloc()
   instrument->name = (char *)strdup("Unnamed");
   instrument->numoutputs = 1;
   instrument->outputs = (struct instrument_output **)calloc(instrument->numoutputs, sizeof(struct instrument_output *));
-  instrument->outputs[0] = (struct instrument_ouput *)calloc(1, sizeof(struct instrument_output));
+  instrument->outputs[0] = (struct instrument_output *)calloc(1, sizeof(struct instrument_output));
   instrument->outputs[0]->defaultvelocity = 127;
 
   return instrument;
