@@ -138,11 +138,7 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 	    if (event->keyval >= '1' && event->keyval <= '9')
 	      channel = event->keyval - '1';
 	    if (channel < editor_song_maxtracks_get(editor)) {
-	      unsigned int mute = editor_song_track_solo_get(editor, channel);
-	      if (mute)
-		editor_song_track_solo_set(editor, channel, 0);
-	      else
-		editor_song_track_solo_set(editor, channel, 1);
+              editor_song_track_solo_set(editor, channel, editor_song_track_solo_get(editor, channel) ? 0 : 1);
 	      gui_trackvolumes_refresh(gui);
 	    }
 	  } else {
@@ -245,11 +241,7 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 	  if (event->keyval >= '1' && event->keyval <= '9')
 	    channel = event->keyval - '1';
 	  if (channel < editor_song_maxtracks_get(editor)) {
-	    unsigned int mute = editor_song_track_mute_get(editor, channel);
-	    if (mute)
-	      editor_song_track_mute_set(editor, channel, 0);
-	    else
-	      editor_song_track_mute_set(editor, channel, 1);
+            editor_song_track_mute_set(editor, channel, editor_song_track_mute_get(editor, channel) ? 0 : 1);
 	    gui_trackvolumes_refresh(gui);
 	  }
 	  break;
@@ -414,26 +406,32 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
       } else if (editor->edit) {
 	switch (tracker->cursor_item) {
 	case 1:
+	case 2:
 	  /* Editing instrument */
 	  if (event->keyval >= '0' && event->keyval <= '9')
 	    data = event->keyval - '0';
-	  else if (event->keyval >= 'a' && event->keyval <= 'z')
+	  else if (event->keyval >= 'a' && event->keyval <= 'f')
 	    data = 10 + event->keyval - 'a';
 	  else if (event->keyval == GDK_Delete)
 	    data = 0;
 
 	  if (data != -1) {
 	    /* Set instrument and refresh */
-	    editor_song_block_current_instrument_set(editor, editor->line, tracker->cursor_ch, data);
+            int ins = editor_song_block_current_instrument_get(editor, editor->line, tracker->cursor_ch);
+            if (tracker->cursor_item == 1)
+              ins = (ins & 0x0f) | (data << 4);
+            else
+              ins = (ins & 0xf0) | data;
+	    editor_song_block_current_instrument_set(editor, editor->line, tracker->cursor_ch, ins);
 	    tracker_redraw_row(tracker, editor->line);
 	    editor_set_line(editor, editor->line + editor->space);
 	    handled = TRUE;
 	  }
 	  break;
-	case 2:
 	case 3:
 	case 4:
 	case 5:
+	case 6:
 	  /* Editing effects */
 	  if (event->keyval >= '0' && event->keyval <= '9')
 	    data = event->keyval - '0';
@@ -444,7 +442,7 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
 
 	  if (data != -1) {
 	    /* Set effect and refresh */
-	    editor_song_block_current_command_set(editor, editor->line, tracker->cursor_ch, editor->cmdpage, tracker->cursor_item - 2, data);
+	    editor_song_block_current_command_set(editor, editor->line, tracker->cursor_ch, editor->cmdpage, tracker->cursor_item - 3, data);
 	    tracker_redraw_row(tracker, editor->line);
 	    editor_set_line(editor, editor->line + editor->space);
 	    handled = TRUE;
@@ -457,21 +455,18 @@ gboolean keyboard_event(GtkWidget * widget, GdkEventKey * event, gboolean presse
       case GDK_Shift_R:
 	/* Right shift: Play block */
 	editor_player_start(editor, MODE_PLAY_BLOCK, 0);
-	gui_statusbar_refresh(gui);
 	handled = TRUE;
 	break;
       case GDK_Control_R:
 	/* Right Control: Play song */
 	editor_player_start(editor, MODE_PLAY_SONG, 0);
-	gui_statusbar_refresh(gui);
 	handled = TRUE;
 	break;
       case GDK_space:
 	/* If the song is playing, stop */
-	if (editor_player_mode_get(editor) != MODE_IDLE) {
+	if (editor_player_mode_get(editor) != MODE_IDLE)
 	  editor_player_stop(editor);
-	  gui_info_refresh(gui);
-	} else {
+	else {
 	  /* Otherwise toggle edit mode */
 	  GtkToggleButton *togglebutton = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gui->xml, "checkbutton_edit"));
 
