@@ -23,7 +23,7 @@
 #  include <config.h>
 #endif
 
-#include <unistd.h>
+#include <time.h>
 #include <libgnome/libgnome.h>
 #include <glade/glade.h>
 #include <glib.h>
@@ -38,7 +38,7 @@ char *nullcheck_error_string = "%s() called with null %s\n";
 int main(int argc, char *argv[])
 {
   struct editor *editor = NULL;
-  int i;
+  const struct timespec nobusywait = { 0, 40000 };
 
   if (!g_thread_supported()) {
     g_thread_init(NULL);
@@ -71,28 +71,30 @@ int main(int argc, char *argv[])
 
       if (!editor->receiveactive) {
         /* Read MIDI input if there is any pending */
-        for (i = 0; i < editor->nummidiinterfaces; i++)
-          while ((data = midi_read_raw(editor->midiinterfaces[i])) != NULL) {
-            editor_midi_input(editor, data);
-            free(data);
-          }
+        while ((data = midi_read_raw(editor->midi)) != NULL) {
+          editor_midi_input(editor, data);
+          free(data);
+        }
       } else {
         /* Receive a MIDI message */
         if (editor_song_message_receive(editor))
           on_button_receive_close_clicked(NULL, editor->gui);          
         else {
           if (editor->receiveautostop)
-            gui_receive_refresh(editor->gui, editor->receivedbytes, 0, FALSE);
+            gui_receive_refresh(editor->gui, editor->receivedbytes, 0);
           else
-            gui_receive_refresh(editor->gui, editor->receivedbytes, editor->song->messages[editor->receivemessage]->length, FALSE);
+            gui_receive_refresh(editor->gui, editor->receivedbytes, editor->song->messages[editor->receivemessage]->length);
         }
       }
+      
+      /* Check whether there have been changes in MIDI interfaces */
+      editor_midi_status_check(editor);
         
       /* Refresh the GUI */
       gui_refresh(editor->gui);
 
       /* Sleep for a while */
-      usleep(40);
+      nanosleep(&nobusywait, 0);
     }
     gdk_threads_leave();
 
