@@ -26,18 +26,21 @@
 #include <time.h>
 #include "tutka.h"
 #include "midi.h"
+#include "player.h"
 
 static char *string_remove_string(char *from, char *toremove);
 static char *string_append_char(char *to, char append);
 static char *string_append_string(char *to, char *append);
 
 /* Open the MIDI subsystem */
-struct midi *midi_open(char *in, char *out, char *unavailablein, char *unavailableout)
+struct midi *midi_open(struct editor *editor, char *in, char *out, char *unavailablein, char *unavailableout)
 {
   struct midi *midi = (struct midi *)calloc(1, sizeof(struct midi));
   snd_seq_addr_t sender, dest;
   snd_seq_port_subscribe_t *subs;
 
+  midi->editor = editor;
+  
   int err = snd_seq_open(&midi->seq, "default", SND_SEQ_OPEN_DUPLEX, SND_SEQ_NONBLOCK);
   if (err < 0) {
     fprintf(stderr, "Couldn't create client: %s", snd_strerror(err));
@@ -153,7 +156,7 @@ unsigned int midi_subscribe(struct midi *midi, int client, int port, unsigned in
 
     /* Try subscribing to the port */
     if (snd_seq_subscribe_port(midi->seq, subs) == 0) {
-      struct midi_alsa *alsa = (struct midi_data *)iface->data;
+      struct midi_alsa *alsa = (struct midi_alsa *)iface->data;
       alsa->subs = subs;
       result = 1;
     }
@@ -188,7 +191,7 @@ unsigned int midi_unsubscribe(struct midi *midi, int client, int port, unsigned
   if (iface == NULL)
     return 0;
 
-  struct midi_alsa *alsa = (struct midi_data *)iface->data;
+  struct midi_alsa *alsa = (struct midi_alsa *)iface->data;
   if (alsa->subs != NULL) {
     snd_seq_addr_t sender, dest;
     snd_seq_port_subscribe_t *subs;
@@ -587,6 +590,22 @@ static inline int midi_read(struct midi *midi, unsigned char *data, unsigned int
     if (snd_seq_event_input(midi->seq, &ev) >= 0) {
       /* Check event type */
       switch (ev->type) {
+      case SND_SEQ_EVENT_START:
+        if (editor_player_get_external_sync(midi->editor) == EXTERNAL_SYNC_MIDI)
+          editor_player_start(midi->editor, MODE_PLAY_SONG, 0);
+        return 0;
+      case SND_SEQ_EVENT_CONTINUE:
+        if (editor_player_get_external_sync(midi->editor) == EXTERNAL_SYNC_MIDI)
+          editor_player_start(midi->editor, MODE_PLAY_SONG, 1);
+        return 0;
+      case SND_SEQ_EVENT_STOP:
+        if (editor_player_get_external_sync(midi->editor) == EXTERNAL_SYNC_MIDI)
+          editor_player_stop(midi->editor);
+        return 0;
+      case SND_SEQ_EVENT_CLOCK:
+        if (editor_player_get_external_sync(midi->editor) == EXTERNAL_SYNC_MIDI)
+          editor_player_external_sync(midi->editor, 1);
+        return 0;
       case SND_SEQ_EVENT_PORT_START:
       case SND_SEQ_EVENT_PORT_EXIT:
       case SND_SEQ_EVENT_PORT_CHANGE:
