@@ -36,26 +36,24 @@
 #include "player.h"
 #include "editor.h"
 
-#define RTC_FREQ 512
+#define MAXIMUM_RTC_FREQ 2048
+#define MINIMUM_RTC_FREQ 512
 
 /* Refreshes playseq from section and block from position */
 static void player_refresh_playseq_and_block(struct player *player)
 {
   nullcheck_void(player, player_refresh_playseq_and_block);
 
-  struct song *song;
-
-  song = player->song;
+  struct song *song = player->song;
 
   if (player->section >= song->numsections)
     player->section = 0;
   player->playseq = song->sections[player->section];
 
-  if (player->position >= song->playseqs[player->playseq]->length) {
+  if (player->position >= song->playseqs[player->playseq]->length)
     player->position = 0;
-    player_refresh_playseq_and_block(player);
-  } else
-    player->block = song->playseqs[player->playseq]->blocknumbers[player->position];
+
+  player->block = song->playseqs[player->playseq]->blocknumbers[player->position];
 }
 
 /* Advances in section and jumps to the beginning if necessary */
@@ -92,48 +90,64 @@ void player_play_note(struct player *player, unsigned int instrument, unsigned c
   int i;
   struct song *song = player->song;
 
-  /* Stop currently playing note */
-  for (i = 0; i < player->trackstatus[track]->numinterfaces; i++) {
-    struct player_interface_status *status = player->trackstatus[track]->interfaces[i];
-    if (status->note != -1) {
-      midi_note_off(player->midi->outputs[i], status->midichannel, status->note, 127);
-      status->note = -1;
+  /* Notes are played if the track is not muted and no tracks are soloed or the current track is soloed */
+  if (!song->tracks[track]->mute && (!player->solo || (player->solo && song->tracks[track]->solo))) {
+    /* Stop currently playing note */
+    for (i = 0; i < player->trackstatus[track]->numinterfaces; i++) {
+      struct player_interface_status *status = player->trackstatus[track]->interfaces[i];
+      if (status->note != -1) {
+	midi_note_off(player->midi->outputs[i], status->midichannel, status->note, 127);
+	status->note = -1;
+      }
+    }
+
+    /* Don't play a note if the instrument does not exist */
+    if (instrument < song->numinstruments && song->instruments[instrument] != NULL) {
+      /* Set new track status (channel, note, volume, time) of each interface defined in the instrument */
+      player->trackstatus[track]->instrument = instrument;
+      for (i = 0; i < song->instruments[instrument]->numoutputs; i++) {
+	struct instrument_output *output = song->instruments[instrument]->outputs[i];
+	struct player_interface_status *status = player->trackstatus[track]->interfaces[output->midiinterface];
+
+	/* Update track status for the selected output */
+	status->note = note + output->transpose;
+	status->midichannel = output->midichannel;
+	status->volume = output->defaultvelocity * volume / 127 * song->tracks[track]->volume / 127 * song->mastervolume / 127;
+	if (output->hold > 0)
+	  status->hold = output->hold;
+	else
+	  status->hold = -1;
+
+	/* Make sure the volume isn't too large */
+	if (status->volume < 0)
+	  status->volume = 127;
+
+	/* Play note */
+	midi_note_on(player->midi->outputs[output->midiinterface], status->midichannel, status->note, status->volume);
+      }
     }
   }
+}
 
-  /* Don't play a note if the instrument does not exist */
-  if (instrument < song->numinstruments && song->instruments[instrument] != NULL) {
-    /* Set new track status (channel, note, volume, time) of each interface defined in the instrument */
-    player->trackstatus[track]->instrument = instrument;
-    for (i = 0; i < song->instruments[instrument]->numoutputs; i++) {
-      struct instrument_output *output = song->instruments[instrument]->outputs[i];
-      struct player_interface_status *status = player->trackstatus[track]->interfaces[output->midiinterface];
+/* Stops notes playing on muted tracks */
+void player_stop_muted(struct player *player)
+{
+  nullcheck_void(player, player_stop_muted);
 
-      /* Apply note transpose */
-      char realnote = note + output->transpose;
-
-      /* Update track status for the selected output */
-      status->note = realnote;
-      status->midichannel = output->midichannel;
-      /* If channels are soloed solo must be taken into account, otherwise not */
-      /* NOTE: Current behaviour does NOT ignore muted or non-soloed channels. The channels are played with volume 0 causing note offs. This is by design. Is it relevant? */
-      if (player->solo)
-	status->volume = output->defaultvelocity * volume / 127 * song->tracks[track]->volume / 127 * song->tracks[track]->solo * (1 - song->tracks[track]->mute) * song->mastervolume / 127;
-      else
-	status->volume = output->defaultvelocity * volume / 127 * song->tracks[track]->volume / 127 * (1 - song->tracks[track]->mute) * song->mastervolume / 127;
-      if (output->hold > 0)
-	status->hold = output->hold;
-      else
-	status->hold = -1;
-
-      /* Make sure the volume isn't too large */
-      if (status->volume < 0)
-	status->volume = 127;
-
-      /* Play note */
-      midi_note_on(player->midi->outputs[output->midiinterface], status->midichannel, status->note, status->volume);
+  struct song *song = player->song;
+  int i, j;
+    
+  for (i = 0; i < song->maxtracks; i++)
+    if (song->tracks[i]->mute || (player->solo && !song->tracks[i]->solo)) {
+      struct player_track_status *trackstatus = player->trackstatus[i];
+      for (j = 0; j < trackstatus->numinterfaces; j++) {
+	struct player_interface_status *interfacestatus = trackstatus->interfaces[j];
+	if (interfacestatus->note != -1) {
+	  midi_note_off(player->midi->outputs[j], interfacestatus->midichannel, interfacestatus->note, 127);
+	  interfacestatus->note = -1;
+	}
+      }
     }
-  }
 }
 
 /* Stops notes playing at the moment */
@@ -191,15 +205,10 @@ static void player_thread(struct player *player)
 
   struct timespec req, rem;
   struct timeval next, now;
+  int prevsched = player->sched;
 
   player->tick = 0;
   player->tickssofar = 0;
-
-  if (player->sched == SCHED_RTC) {
-    if (ioctl(player->rtc, RTC_UIE_OFF, 0) == -1 || ioctl(player->rtc, RTC_IRQP_SET, RTC_FREQ) == -1 || ioctl(player->rtc, RTC_PIE_ON, 0) == -1)
-      player->sched = SCHED_NANOSLEEP;
-  }
-
   next.tv_sec = player->playingstarted.tv_sec;
   next.tv_usec = player->playingstarted.tv_usec;
 
@@ -211,15 +220,24 @@ static void player_thread(struct player *player)
     unsigned char postcommand = 0, postvalue = 0;
     int looped = 0;
 
-    /* Calculate time of next tick (tick every 1000000/((BPM/60)*24) usecs) */
-    next.tv_sec += (25 / player->song->tempo) / 10;
-    next.tv_usec += ((2500000 / player->song->tempo) % 1000000);
-    while (next.tv_usec > 1000000) {
-      next.tv_sec++;
-      next.tv_usec -= 1000000;
-    }
+    /* Lock */
+    if (player->mutex != NULL)
+      g_mutex_lock(player->mutex);
+
+    if (player->sched == SCHED_RTC || player->sched == SCHED_NANOSLEEP) {
+      /* If the scheduler has changed from external sync reset the timing */
+      if (prevsched == SCHED_EXTERNAL_SYNC)
+	gettimeofday(&next, NULL);
+      prevsched = player->sched;
+
+      /* Calculate time of next tick (tick every 1000000/((BPM/60)*24) usecs) */
+      next.tv_sec += (25 / player->song->tempo) / 10;
+      next.tv_usec += ((2500000 / player->song->tempo) % 1000000);
+      while (next.tv_usec > 1000000) {
+	next.tv_sec++;
+	next.tv_usec -= 1000000;
+      }
 
-    if (player->sched != SCHED_NONE) {
       gettimeofday(&now, NULL);
 
       /* Select scheduling type */
@@ -233,11 +251,23 @@ static void player_thread(struct player *player)
 	}
 
 	/* Sleep until the next tick if necessary */
+	if (player->mutex != NULL)
+	  g_mutex_unlock(player->mutex);
 	if (req.tv_sec >= 0)
 	  while (nanosleep(&req, &rem) == -1);
+	if (player->mutex != NULL)
+	  g_mutex_lock(player->mutex);
       } else {
+	/* Make sure periodic interrupts are on */
+	if (!player->rtc_pie) {
+	  ioctl(player->rtc, RTC_PIE_ON, 0);
+	  player->rtc_pie = 1;
+	}
+
 	/* Rtc: read RTC interrupts until it's time to play again */
-	while ((next.tv_sec - now.tv_sec) > 0 || ((next.tv_sec - now.tv_sec) == 0 && (next.tv_usec - now.tv_usec) > 1000000 / RTC_FREQ)) {
+	if (player->mutex != NULL)
+	  g_mutex_unlock(player->mutex);
+	while ((next.tv_sec - now.tv_sec) > 0 || ((next.tv_sec - now.tv_sec) == 0 && (next.tv_usec - now.tv_usec) > 1000000 / player->rtc_frequency)) {
 	  unsigned long l;
 	  read(player->rtc, &l, sizeof(unsigned long));
 	  gettimeofday(&now, NULL);
@@ -245,12 +275,18 @@ static void player_thread(struct player *player)
 	while ((next.tv_sec - now.tv_sec) > 0 || ((next.tv_sec - now.tv_sec) == 0 && (next.tv_usec - now.tv_usec) > 0)) {
 	  gettimeofday(&now, NULL);
 	}
+	if (player->mutex != NULL)
+	  g_mutex_lock(player->mutex);
       }
-    }
-
-    /* Lock */
-    if (player->mutex != NULL)
-      g_mutex_lock(player->mutex);
+    } else if (player->sched == SCHED_EXTERNAL_SYNC) {
+      prevsched = player->sched;
+      if (player->external_sync_ticks == 0)
+	/* Wait for a sync signal to come in */
+	g_cond_wait(player->external_sync, player->mutex);
+      if (player->external_sync_ticks > 0)
+        player->external_sync_ticks--;
+    } else
+      prevsched = player->sched;
 
     /* Handle this tick */
     for (i = 0; i < player->midi->numoutputs; i++)
@@ -266,13 +302,14 @@ static void player_thread(struct player *player)
       for (i = 0; i < player->midi->numoutputs; i++)
 	midi_clock(player->midi->outputs[i]);
 
-    /* Pass 1: Stop notes if there are new notes about to be played */
+    /* Stop notes if there are new notes about to be played */
     for (i = 0; i < b->tracks; i++) {
       struct player_track_status *trackstatus = player->trackstatus[i];
       int delay = 0;
       unsigned char note = b->notes[(b->tracks * player->line + i) * 2];
 
-      if (note != 0) {
+      /* Notes are played if the track is not muted and no tracks are soloed or the current track is soloed */
+      if (note != 0 && !song->tracks[i]->mute && (!player->solo || (player->solo && song->tracks[i]->solo))) {
 	for (j = 0; j < b->commandpages; j++) {
 	  unsigned char command = b->commands[j * b->tracks * b->length * 2 + (b->tracks * player->line + i) * 2];
 	  unsigned char value = b->commands[j * b->tracks * b->length * 2 + (b->tracks * player->line + i) * 2 + 1];
@@ -309,7 +346,7 @@ static void player_thread(struct player *player)
       }
     }
 
-    /* Pass 2: Play notes scheduled to be played */
+    /* Play notes scheduled to be played */
     for (i = 0; i < b->tracks; i++) {
       struct player_track_status *trackstatus = player->trackstatus[i];
       unsigned int volume = 127;
@@ -665,8 +702,11 @@ static void player_thread(struct player *player)
       player->playedsofar.tv_usec -= 1000000;
     }
 
-    if (player->sched == SCHED_RTC)
+    /* Turn off periodic interrupts */
+    if (player->sched == SCHED_RTC && player->rtc_pie) {
       ioctl(player->rtc, RTC_PIE_OFF, 0);
+      player->rtc_pie = 0;
+    }
 
     /* Stop all playing notes */
     player_stop_notes(player);
@@ -686,6 +726,7 @@ void player_midi_export(struct song *song, struct midi_interface *midi)
 {
   int i, j;
   struct player *player = (struct player *)calloc(1, sizeof(struct player));
+  player->midi = (struct midi *)calloc(1, sizeof(struct midi));
 
   /* Check how many MIDI interfaces are used in the song */
   for (i = 0; i < song->numinstruments; i++) {
@@ -743,7 +784,7 @@ void player_midi_export(struct song *song, struct midi_interface *midi)
 }
 
 /* Creates a player for a song */
-struct player *player_open(struct song *song, struct editor *editor, struct midi *midi, unsigned int scheduler)
+struct player *player_open(struct song *song, struct editor *editor, struct midi *midi)
 {
   struct player *player = (struct player *)calloc(1, sizeof(struct player));
   int i, j;
@@ -751,16 +792,9 @@ struct player *player_open(struct song *song, struct editor *editor, struct midi
   player->song = song;
   player->editor = editor;
   player->midi = midi;
-  player->sched = scheduler;
-  if (player->sched == SCHED_RTC) {
-    player->rtc = open("/dev/rtc", O_RDONLY);
-    if (player->rtc != -1)
-      player->sched = SCHED_RTC;
-    else
-      player->sched = SCHED_NANOSLEEP;
-  } else
-    player->rtc = -1;
+  player->sched = SCHED_NANOSLEEP;
   player->mutex = g_mutex_new();
+  player->external_sync = g_cond_new();
 
   /* Initialize track status array */
   player_trackstatus_create(player, 0);
@@ -798,6 +832,10 @@ void player_close(struct player *player)
   if (player->mutex != NULL)
     g_mutex_free(player->mutex);
 
+  /* Free the cond */
+  if (player->external_sync != NULL)
+    g_cond_free(player->external_sync);
+
   /* Free MIDI controller values array */
   if (player->midicontrollervalues) {
     for (i = 0; i < player->nummidicontrollervalues; i++)
@@ -859,6 +897,17 @@ void player_start(struct player *player, unsigned int mode, int section, int pos
   else
     player_reset_time(player, TRUE);
 
+  /* Tell the editor about the new position if an editor exists */
+  if (player->editor != NULL) {
+    editor_set_section(player->editor, player->section);
+    editor_set_playseq(player->editor, player->playseq);
+    editor_set_position(player->editor, player->position);
+    editor_set_block(player->editor, player->block);
+    editor_set_line(player->editor, player->line);
+    if (player->sched != SCHED_NONE)
+      editor_set_time(player->editor, (int)(player->playedsofar.tv_sec * 1000 + player->playedsofar.tv_usec / 1000) / 1000);
+  }
+
   /* Create a new thread only if there isn't one already */
   if (player->thread == NULL) {
     player->thread = g_thread_create((GThreadFunc) player_thread, player, TRUE, NULL);
@@ -875,9 +924,16 @@ void player_stop(struct player *player)
     player->mode = MODE_IDLE;
 
   if (player->thread != NULL) {
+    /* Mark the thread for killing */
     g_mutex_lock(player->mutex);
     player->killthread = 1;
     g_mutex_unlock(player->mutex);
+    
+    /* If external sync is used send sync to get out of the sync wait loop */
+    if (player->sched == SCHED_EXTERNAL_SYNC)
+      player_external_sync(player, 0);
+    
+    /* Wait until the thread is dead */
     g_thread_join(player->thread);
 
     player->thread = NULL;
@@ -958,14 +1014,16 @@ void player_set_position(struct player *player, int position)
 {
   nullcheck_void(player, player_set_position);
 
+  g_mutex_lock(player->mutex);
+
   /* Bounds checking */
   if (position < 0)
     position = 0;
   if (position >= player->song->playseqs[player->playseq]->length)
     position = player->song->playseqs[player->playseq]->length - 1;
 
-  g_mutex_lock(player->mutex);
   player->position = position;
+  
   g_mutex_unlock(player->mutex);
 }
 
@@ -990,17 +1048,29 @@ void player_set_line(struct player *player, int line)
 {
   nullcheck_void(player, player_set_line);
 
+  g_mutex_lock(player->mutex);
+
   /* Bounds checking */
   while (line < 0)
     line += player->song->blocks[player->block]->length;
   while (line >= player->song->blocks[player->block]->length)
     line -= player->song->blocks[player->block]->length;
 
-  g_mutex_lock(player->mutex);
   player->line = line;
+
   g_mutex_unlock(player->mutex);
 }
 
+/* Set current tick */
+void player_set_tick(struct player *player, int tick)
+{
+  nullcheck_void(player, player_set_tick);
+  
+  g_mutex_lock(player->mutex);
+  player->tick = tick;
+  g_mutex_unlock(player->mutex);  
+}
+
 /* Resets the player time */
 void player_reset_time(struct player *player, gboolean reset_sofar)
 {
@@ -1079,7 +1149,8 @@ void player_midi_changed(struct player *player)
 }
 
 /* Lock the player */
-void player_lock(struct player *player) {
+void player_lock(struct player *player)
+{
   nullcheck_void(player, player_lock);
 
   if (player->mutex != NULL)
@@ -1087,9 +1158,70 @@ void player_lock(struct player *player) {
 }
 
 /* Unlock the player */
-void player_unlock(struct player *player) {
+void player_unlock(struct player *player)
+{
   nullcheck_void(player, player_unlock);
 
   if (player->mutex != NULL)
     g_mutex_unlock(player->mutex);
 }
+
+/* A method to notify the player about an incoming sync signal */
+void player_external_sync(struct player *player, unsigned int ticks)
+{
+  nullcheck_void(player, player_external_sync);
+
+  if (player->mutex != NULL) {
+    g_mutex_lock(player->mutex);
+    if (player->mode != MODE_IDLE)
+      player->external_sync_ticks += ticks;
+    if (player->external_sync != NULL)
+      g_cond_signal(player->external_sync);
+    g_mutex_unlock(player->mutex);
+  }
+}
+
+/* Set the scheduler of a player */
+void player_set_scheduler(struct player *player, unsigned int scheduler)
+{
+  nullcheck_void(player, player_set_scheduler);
+
+  if (player->mutex != NULL)
+    g_mutex_lock(player->mutex);
+
+  player->sched = scheduler;
+  if (player->sched == SCHED_RTC) {
+    player->rtc = open("/dev/rtc", O_RDONLY);
+    if (player->rtc == -1)
+      player->sched = SCHED_NANOSLEEP;
+    else {
+      gboolean success = FALSE;
+      player->rtc_frequency = MAXIMUM_RTC_FREQ;
+      player->rtc_pie = 0;
+
+      /* RTC scheduling requested: try every 2^N value down from MAXIMUM_RTC_FREQ to MINIMUM_RTC_FREQ */
+      while (player->rtc_frequency >= MINIMUM_RTC_FREQ) {
+	if (ioctl(player->rtc, RTC_UIE_OFF, 0) == -1 || ioctl(player->rtc, RTC_IRQP_SET, player->rtc_frequency) == -1)
+	  player->rtc_frequency /= 2;
+	else {
+	  success = TRUE;
+	  break;
+	}
+      }
+
+      /* If all frequencies failed fall back to nanosleep() */
+      if (!success) {
+	close(player->rtc);
+	player->rtc = -1;
+	player->sched = SCHED_NANOSLEEP;
+      }
+    }
+  } else {
+    if (player->rtc != -1)
+      close(player->rtc);
+    player->rtc = -1;
+  }
+
+  if (player->mutex != NULL)
+    g_mutex_unlock(player->mutex);
+}
