@@ -31,11 +31,11 @@ char *nullcheck_error_string = "%s() called with null %s\n";
 
 int main(int argc, char *argv[])
 {
-  struct midi *midi;
+  struct midi_interface *midi;
   int i, j;
 
   midi = midi_alloc(MIDI_OSS, "/dev/midi", NULL);
-  midi_open(midi, DIRECTION_OUT);
+  midi_interface_open(midi, DIRECTION_OUT);
   
   for (i = 0; i < 16; i++)
     for (j = 0; j < 128; j++)
