@@ -116,6 +116,26 @@ void editor_song_lock(struct editor *editor)
 void editor_song_unlock(struct editor *editor)
 {
 }
+void editor_player_start(struct editor *editor, unsigned int mode, int cont)
+{
+}
+void editor_player_stop(struct editor *editor)
+{
+}
+void editor_player_external_sync(struct editor *editor, unsigned int ticks)
+{
+}
+unsigned int editor_player_get_external_sync(struct editor *editor)
+{
+  return 0;
+}
+unsigned int editor_player_available(struct editor *editor)
+{
+  return 1;
+}
+void editor_seek(struct editor *editor, int ticks)
+{
+}
 
 int main(int argc, char *argv[])
 {
@@ -145,10 +165,11 @@ int main(int argc, char *argv[])
 
   if (editor->song != NULL) {
     /* Open MIDI */
-    editor->midi = midi_open(gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/in", NULL), gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/out", NULL), gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailablein", NULL), gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailableout", NULL));
+    editor->midi = midi_open(editor, gconf_client_get_string(editor->gconfclient, "/apps/tutka/midi_interfaces/in", NULL), gconf_client_get_string(editor->gconfclient, "/apps/tutka/midi_interfaces/out", NULL), gconf_client_get_string(editor->gconfclient, "/apps/tutka/midi_interfaces/unavailable_in", NULL), gconf_client_get_string(editor->gconfclient, "/apps/tutka/midiinterfaces/unavailable_out", NULL));
 
     /* Open the player and start playing */
-    editor->player = player_open(editor->song, editor, editor->midi, gconf_client_get_int(editor->gconfclient, "/apps/tutka/scheduler", NULL));
+    editor->player = player_open(editor->song, editor, editor->midi);
+    player_set_scheduler(editor->player, gconf_client_get_int(editor->gconfclient, "/apps/tutka/scheduling_mode", NULL));
     editor->player->sched = SCHED_RTC;
     player_start(editor->player, MODE_PLAY_SONG, 0, 0, 0, 0);
 
